function [X,w,info] = BCDsolver(X0,w0,gpcmodel,BCDoptions)
%   To solve the nonlinear least square subproblem by BCD
    tic;
    if isfield(BCDoptions,'maxiter')
        maxiter = BCDoptions.maxiter;
    else
        maxiter = 1000;
    end
    if isfield(BCDoptions,'eps')
        BCDeps = BCDoptions.eps;
    else
        BCDeps = 1e-6;
    end
    info.success = 0;
    
    X = X0;
    w = w0;
    rhs = zeros(gpcmodel.nbasis_2p,1);rhs(1)=1;
    Phi = eval_Phi(X0,gpcmodel,2*gpcmodel.order);
    for iter = 1:maxiter 
        options1 = optimset('Display','off');
        w = lsqnonneg(Phi, rhs,options1); 
        J = objgrad(X, size(Phi,1),w,gpcmodel);
        r =  Phi*w - rhs; 
        [dx,flag] = lsqr(J, -r);  
        X = X + reshape(dx,size(X));
        Phi = eval_Phi(X,gpcmodel,2*gpcmodel.order);
        res = norm(Phi*w-rhs);
        if res < BCDeps
            info.success = 1;
            break
        end
    end
    info.time = toc;
    info.res = res;
    info.iter = iter;
    
    %% end of the main function
    
    function gradx = objgrad(X,nbasis, w,model)
    % computing the gradient of Least Square objective function
         
        [nsmp,dim] = size(X);  
        
        dX = zeros(nbasis,dim,nsmp); 
        for i = 1:nsmp
            dX(:,:,i)= w(i)*grad_basis_onesample(X(i,:),model,2*model.order);
        end  
        gradx = reshape(permute(dX,[1 3 2]),nbasis,nsmp*dim);
        
    end

    function grad_basis = grad_basis_onesample(sample,gPCmodel,order)
    %   the gradient of basis functions with respect to one sample 
        d = gPCmodel.dim;
        assert(length(sample)==d)
        if order == gPCmodel.order
            BasisCoeff=gPCmodel.BasisCoeff;
        elseif order == 2*gPCmodel.order
            BasisCoeff=gPCmodel.BasisCoeff_2p;
        end
        N_order = size(BasisCoeff,2);
        grad_monmial = zeros(N_order,d);

        % order  = 1;
        grad_monmial(2:d+1,:) = eye(d);
        cind = d+2; 
        % order >= 2; 
        for n = 2:order
            pow = monomial_pow(n,d);
            cnum = size(pow,1);
            tmp = zeros(cnum,d);
            aycnum = [1:cnum]';
            for i = 1:n 
               smp = sample(pow(:,[1:i-1,i+1:n]));
               if n >=3
                   smp = prod(smp,2);
               end
               tmp(aycnum+(pow(:,i)-1)*cnum) =  tmp(aycnum+(pow(:,i)-1)*cnum) + smp(:);
            end
            grad_monmial(cind:cind+cnum-1,:) = tmp;
            cind = cind + cnum;
        end 

        grad_basis = BasisCoeff*grad_monmial;
    end


    function pow  = monomial_pow(n,d)
    %	monomials of all polynomials with dimension = d, order = n
    %	in graded lexicographic order  
        pow = nchoosek(1:n-1+d,n);
        for j = 1:n
            pow(:,j) = pow(:,j)-j+1;
        end   
    end
end