function [X,w] = ClusteredSampling(GMobj,gpcmodel,nsamples)
%   Clustered Sampling
    nsamples_MC = max(5*nsamples, gpcmodel.nbasis_2p+20);
    XMC = MCSampling(GMobj,nsamples_MC);
    Phi = eval_Phi(XMC,gpcmodel,2*gpcmodel.order); 
    m = zeros(gpcmodel.nbasis_2p,1);m(1) = 1; 
    wMC = lsqnonneg(Phi, m);
    [X,w] = WCLCluster(XMC,wMC,nsamples); 
    
    %%  end of the main function
    
    function [Xh,Wh,label] = WCLCluster(X0,w0,nCluster)
    %   weighted complete linkage clustering

        S1 = w0>0;
        X0 = X0(S1,:);
        w0 = w0(S1);
        [S, d] = size(X0);
        if nCluster>S
            Wh = w0;
            Xh = X0;
            return;
        end

        label = 1:S; 
        Wt = w0;
        Dist = inf*ones(S,S);
        for i= 2:S
            for j = 1:i-1
                Dist(i,j) = norm(X0(i,:)-X0(j,:));
            end
        end 

        while S > nCluster 
            [A,B] = find2NearPoint(Dist, Wt) ;
            [Dist,label,Wt] = update(A,B,Dist,label,Wt); 
            S = S-1;
        end
        Wh = Wt(Wt>0); 
        M0 = length(Wh);
        Xh = zeros(M0,d);
        for i = 1:M0
            indi = label==i;
            Xh(i,:) = Xh(i,:) + w0(indi)'*X0(indi,:)/Wh(i); 
        end  
    end
    
    function [A,B] = find2NearPoint(Dist,Wt)  
        % A>=B
        sz = size(Dist,1);
        %Dist1 = Dist;
        Dist1 = Dist.*(repmat(Wt,1,sz) + repmat(Wt',sz,1));
        minv = inf;
        for is = 2:sz
            [v,ind] = min(Dist1(is,:));
            if v<minv
                A = is;
                B = ind;
                minv = v;
            end
        end
    end

    function [Dist,label,Wt] = update(A,B,Dist,label,Wt) 
        if B>A
            tmp = B;
            B = A;
            A = tmp;
        end
        distA = [Dist(A,1:A-1)';0;Dist(A+1:end,A)];
        distB = [Dist(B,1:B-1)';0;Dist(B+1:end,B)];
        distAB = max(distA, distB);%complete linkage
        %distAB =  (Wt(A)*distA + Wt(B)*distB)/(Wt(A) + Wt(B));
        % average linkage
        Dist(B,1:B-1) = distAB(1:B-1);
        Dist(B+1:end,B) = distAB(B+1:end);
        
        ind = [1:A-1,A+1:length(Wt)];
        Dist = Dist(ind, ind); 
        Wt(B) = Wt(A) + Wt(B);
        Wt = Wt(ind); 
        
        % B<A
        label(label == A) = B;
        label(label > A) = label(label > A) - 1;  
    end
 
 

end