function [gpcmodel,info] = SCQuadrature(GMobj,gpcmodel,SCoptions)
%   To compute the quadrature rule in stochastic collocation 

    Np = gpcmodel.nbasis;
    N2p = gpcmodel.nbasis_2p;
    if isfield(SCoptions,'n_QPoints')
        n_QPoints = SCoptions.n_QPoints;
    else
        n_QPoints = min(Np*2,floor(1/2*(Np+N2p)));
    end
    if isfield(SCoptions,'eps')
        eps = SCoptions.eps;
    else
        eps = 1e-6;
    end
    nsmp_add = floor(gpcmodel.nbasis*0.1)+1;

    
    %   To initialize the quadrature points by Cluster
    [Xk,wk] = ClusteredSampling(GMobj,gpcmodel,n_QPoints);
    %   To improve the quadrature points
    Xopt = [];
    wopt = [];
    BCDoptions.eps = eps;
    % Increase phase
    while 1
        [Xk,wk,info] = BCDsolver(Xk,wk,gpcmodel,BCDoptions); 
        if info.success == 1 
            break
        else
            [Xk,wk] = increase_samples(GMobj,gpcmodel,length(wk)+nsmp_add);
        end
        fprintf(' Increase phase, M =%3d, res =%4.2e\n',length(wk),info.res);
    end 
    % Decrease phase
    while 1
        Xopt = Xk;
        wopt = wk;
        [Xk,wk] = decrease_samples(Xk,wk);
        [Xk,wk,info] = BCDsolver(Xk,wk,gpcmodel,BCDoptions);
        fprintf(' Decrease phase, M =%3d, res =%4.2e\n',length(wk),info.res);
        if info.success == 0
            break;
        end
    end
    
    % output 
    gpcmodel.QPoints = Xopt;
    gpcmodel.QWeights = wopt; 
    
    %%  end of the main 
    
    function [Xh,wh] = increase_samples(obj,model,nsample) 
    %   increase sample points
        [Xh, wh] = ClusteredSampling(obj,model,nsample); 
    end
    
    function [X,w] = decrease_samples(X,w)
    %   decrease sample  points  
        [~, minloc] = min(w); 
        ind = [1:minloc-1, minloc+1:length(w)];
        w = w(ind);
        w = w/sum(w);
        X = X(ind,:);
    end
end