function model = gPCmodel(p,GMobj)
%   To initialize the generated Polynomial Chaos model
    model.order = p;
    model.dim = GMobj.dim;
    model.nbasis = nchoosek(p+model.dim,p);
    model.nbasis_2p = nchoosek(2*p+model.dim,2*p);
    
    BasisCoeff = gen_BasisCoeff(2*p,GMobj);
    model.BasisCoeff_2p = BasisCoeff;
    model.BasisCoeff = BasisCoeff(1:model.nbasis,1:model.nbasis);
 
    %% end of the main function
 
    function BasisCoeff = gen_BasisCoeff(order_2p,GMobj)
    %   To generate the Coefficient of Basis functions via Gram-Schmidt 
        moment_4p = moment_tt_close(2*order_2p,GMobj);
        d = GMobj.dim; 
        N_2p = nchoosek(order_2p+d,d); 
        covMat = covMAT(order_2p,d,moment_4p); 

        BasisCoeff = zeros(N_2p,N_2p); 
        BasisCoeff(1,1)=1;
        % Gram-schmidt
        MtM = zeros(N_2p,N_2p);
        MtM(1,1)=1;
        for j=2:N_2p
            cj = [-covMat(j,1:j-1)*MtM(1:j-1,1:j-1) 1]; % new coefficient
            norm_phi = sqrt(cj*covMat(1:j,1:j)*cj');    % normlization
            cnew = cj/ norm_phi;
            BasisCoeff(j,1:j) = cnew; 
            MtM(1:j,1:j) = MtM(1:j,1:j) + cnew'*cnew;
        end
    end 
    
    function covMat = covMAT(norder,d,m2) 
    %   To compute the covariance matrix
       Nd = nchoosek(norder+d,d)-1;
       alp = zeros(Nd,d);
       cind = 1;
       for i = 1:norder
           cnum = nchoosek(i+d-1,i); 
           alp(cind:cind+cnum-1,:) = monomial_alp(i,d);
           cind = cind+cnum;
       end
       covMat = zeros(Nd,Nd);
       for i = 1:Nd  
           covMat(i,1:i) = alp2ind(repmat(alp(i,:),i,1)+alp(1:i,:)); 
       end
       covMat = m2(covMat + covMat'-diag(diag(covMat)));  
       covMat = [1 m2(1:Nd)';m2(1:Nd) covMat];
    end 
 
    function Ind = alp2ind(Alpha)
    % computing the coressponding index in the set of summation
        
        [nd , d] = size(Alpha);
        Ind = zeros(nd,1);
        for k = 1:nd
            alpha = Alpha(k,:);
            n = sum(alpha);
            ind = nchoosek(n-1+d,d)-1;

            for i = 1:d
                if alpha(i) == n
                    ind = ind + 1;
                    break;
                else
                    ind = ind + nchoosek(n-alpha(i)-1+d-i,d-i);
                    n = n - alpha(i);
                end
            end
            Ind (k) = ind;
        end

    end
    function  alpha = monomial_alp(n,d)
    % the monomials of all polynomials with dimension = d, and order = n
    
        pow = nchoosek(1:n-1+d,n);
        for j = 1:n
            pow(:,j) = pow(:,j)-j+1;
        end 
        
        dimn = size(pow,1);
        alpha = zeros(dimn,d); 
        for j = 1:n
            for k = 1:dimn
                alpha(k,pow(k,j)) = alpha(k,pow(k,j)) + 1;
            end
        end 
    end
end