function mom = moment_tt_close(norder, gmobj)
%	the close form solution for all moments <=norder
    nmix = gmobj.nmix;
    sigma = gmobj.sigma;
    A = sigma;
    mu = gmobj.mu;
    w = gmobj.w; 
    for ia = 1:nmix
       A(:,:,ia) = chol(sigma(:,:,ia),'lower');
    end
    nstoreTT = floor(norder/2);
    mom = TTmom(A, mu, w,1,norder,nstoreTT);
 
    %% end of the main function  
        
    function mom2 = TTmom(A,mu,p,ordermin,ordermax,nstoreTT)
    %   Construct the FTT and then calculate the moments
        TTall = TTstruct(nstoreTT,A,mu); 
        Indmatall = cell(ordermax+1,ordermax+1);
        for ii= 1:ordermax
            tmp = imgmat(ii);
            Indmatall{ii+1,ii+1} = sparse(tmp); 
            for jj = 0:ii-1
                Indmatall{ii+1,jj+1} = tmp(:,1:2^jj);
            end
        end 
        dim = size(mu,2);
        mom2= [];
        for od = ordermin:ordermax
            tp = zeros(nchoosek(dim+od-1,od),1); 
            pow = monomial_pow(od,dim);      
            for ip= 1:size(pow,1)
               powi = pow(ip,:); 
               spow = splitpow(powi,nstoreTT); 
               tp(ip) = innprod(TTall((spow(:,2)-1)*dim + spow(:,1)),Indmatall,p,spow);  
            end
            mom2 = [mom2; tp];
        end
    end

    %   To construct the TT for all monimials
    function TTall = TTstruct(order,A,mu) 
        [dim,~,nm] = size(A);
        TTall = cell(dim,order);
        for di = 1:dim  
            Am1 = cell(di+1,nm);% TT factors
            for m = 1:nm 
                Am = A(:,:,m);    
                Am1{1,m} = [mu(m,di) 1]; 
                for idi = 1:di-1 
                    Am1{idi+1,m} = [1 0; Am(di,idi) 1];
                end 
                Am1{di+1,m} = [1; Am(di,di)];
            end
            TTall(di,1) = {Am1};  
            
            Amold = Am1;
            for od = 2:order  
                Am2 = cell(di+1,nm);
                for m = 1:nm  
                    for idi = 1:di+1
                        Am2{idi,m} = sparse(kron(Amold{idi,m},Am1{idi,m})); 
                    end  
                end
                TTall(di,od) = {Am2};  
                Amold = Am2;
            end 
        end
    end 
    %   To split alpha into alpha_1+alpha_2
    function spow = splitpow(pow, maxord)
        powi = [];
        spow = zeros(length(pow),2); 
        nsp = 0;
        for is = 1:length(pow)
            if isempty(powi)
                powi = pow(is);
                irep = 1;
            else
                if pow(is) == powi
                    irep = irep+1;
                    if irep == maxord
                        nsp = nsp+1;
                        spow(nsp,:) = [powi, irep];  
                        powi = [];
                    end
                else
                    nsp = nsp+1;
                    spow(nsp,:) = [powi, irep];  
                    powi = pow(is);
                    irep = 1;
                end
            end
        end
        
        if ~isempty(powi)
            nsp = nsp+1;
            spow(nsp,:) = [powi, irep]; 
        end
        spow = spow(1:nsp,:);      
    end

    % contraction of all tensor train 
    function tp = innprod(TTm,Indmatm,p,powi)    
        numTT = size(powi,1);
        mix = length(p);
        % y increasing order 
        [lenTT, indTT] = sort(powi(:,1)+1); 
        ordTT = powi(indTT,2);
        
        TTm2 = TTm;
        TTm = cell(max(lenTT),numTT,mix); 
        for iT = 1:numTT    
            TTm(1:lenTT(iT),iT,:) = TTm2{indTT(iT)};   
        end 
        
        clear TTm2 indTT
        
        % to compute the value   
        tp = 0;
        
        for im = 1:length(p) 
            irow = 1;
            tpp = krons(TTm(irow,:,im));  
            od = sum(ordTT(1:end))+1;
            iT = 1;
            while iT <=numTT
                for iT2 = irow+1:lenTT(iT)-1 
                    tpp = tpp*(krons(TTm(iT2,iT:end,im)).*Indmatm{od,od});  
                end
                
                % find next iT with different ord
                odnew = [];
                for iTnew = iT+1:numTT
                    if lenTT(iTnew) == lenTT(iT)
                        continue; 
                    else 
                        odnew = sum(ordTT(iTnew:end))+1; 
                        break
                    end
                end   
                if isempty(odnew)
                    odnew = 1;
                    iTnew = numTT + 1;
                end  
                irow = lenTT(iT);
                tpp = tpp*(krons(TTm(irow,iT:end,im)).*Indmatm{od,odnew}); 
                od = odnew;
                iT = iTnew;
            end
            
            
            tp = tp + p(im)*tpp;
        end 
    end

    % kron of several matrix
    function mk = krons(As)
        mk = As{1};
        for is = 2:length(As)
            mk = kron(mk, As{is}); 
        end
    end
  
    function immat2 = imgmat(dim) 
        if dim < 1
            immat2 = 1;
            return
        end
        immat0 = [1 0;2  1];
        immat = immat0;
        for t = 2:dim
            immat = sparse(kron(immat,immat0));
        end
        immat2 = immat;
        Ind = immat>1;
        immat2(Ind) = log(immat(Ind))/log(2);
        Ind_odd = rem(immat2,2)==1;
        Ind_even = Ind-Ind_odd>0;
        immat2(Ind_odd) = 0;
        immat2(Ind_even) = fact2(immat2(Ind_even)-1);   
        immat2(immat==1)=1;
    end 

    function f2 = fact2(v)
       f2 = ones(length(v),1);
       for f =1:length(v) 
           f2(f) = prod(1:2:v(f));
       end
    end

    function pow  = monomial_pow(n,d)
        pow = nchoosek(1:n-1+d,n);
        for j = 1:n
            pow(:,j) = pow(:,j)-j+1;
        end  
        
    end

    function Ind = alp2ind(Alpha)
    % computing the coressponding index in the set of summation
        
        [nd , d] = size(Alpha);
        Ind = zeros(nd,1);
        for k = 1:nd
            alpha = Alpha(k,:);
            n = sum(alpha);
            ind = nchoosek(n-1+d,d)-1;

            for i = 1:d
                if alpha(i) == n
                    ind = ind + 1;
                    break;
                else
                    ind = ind + nchoosek(n-alpha(i)-1+d-i,d-i);
                    n = n - alpha(i);
                end
            end
            Ind (k) = ind;
        end

    end
end