%   The Gaussian mixture distribution
clear
d = 2; 
nmix = 2;
w = ones(nmix,1)/nmix;
mu(1,:) = ones(1,d); mu(2,:) = -ones(1,d);
Sigma = zeros(d,d,nmix);
for i = 1:nmix 
    Sigma(:,:,i) = eye(d)/10 + ones(d)/100;
end 

GMobj = GMdistribution(mu,Sigma,w);

%   Initialize the gPC model + basis functions
p = 3;
gpcmodel = gPCmodel(p,GMobj);

%   Compute the quadratue points and weights
SCoptions.eps = 1e-8;
[gpcmodel,info] = SCQuadrature(GMobj,gpcmodel,SCoptions);

%   Compute the gPC coefficient
fun_synthetic = @(x)(exp(x(:,1)/10+1)+0.1*cos(x(:,1)/10+1).*sin(x(:,2)/10+2));
y_Qpoints = fun_synthetic(gpcmodel.QPoints);
gpcmodel = gPCCoeff(gpcmodel, y_Qpoints);

%   Generate MC points and check errors of gPC
nsamples_MC = 1e5;
x_MC = MCSampling(GMobj,nsamples_MC);
y_MC = fun_synthetic(x_MC);

y_gPC = gPCSimulator(x_MC, gpcmodel);

fprintf('Relative approx err = %f\n', norm(y_MC-y_gPC)/ norm(y_MC));
 

